@@ -4,8 +4,8 @@
  *
  * Copyright (C) 2004 Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2024 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2024 Milan Broz
+ * Copyright (C) 2009-2025 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2025 Milan Broz
  */
 
 #ifndef INTERNAL_H
@@ -48,24 +48,36 @@
 
 struct crypt_device;
 struct luks2_reencrypt;
+struct volume_key;
 
-struct volume_key {
-	int id;
-	size_t keylength;
-	const char *key_description;
-	struct volume_key *next;
-	char key[];
-};
+typedef enum {
+	KEY_QUALITY_KEY = 0,
+	KEY_QUALITY_NORMAL,
+	KEY_QUALITY_EMPTY
+} key_quality_info;
 
 struct volume_key *crypt_alloc_volume_key(size_t keylength, const char *key);
-struct volume_key *crypt_generate_volume_key(struct crypt_device *cd, size_t keylength);
+struct volume_key *crypt_alloc_volume_key_by_safe_alloc(void **safe_alloc);
+struct volume_key *crypt_generate_volume_key(struct crypt_device *cd, size_t keylength,
+					     key_quality_info quality);
 void crypt_free_volume_key(struct volume_key *vk);
-int crypt_volume_key_set_description(struct volume_key *key, const char *key_description);
+const char *crypt_volume_key_get_key(const struct volume_key *vk);
+size_t crypt_volume_key_length(const struct volume_key *vk);
+int crypt_volume_key_set_description(struct volume_key *key,
+				     const char *key_description, key_type_t keyring_key_type);
+int crypt_volume_key_set_description_by_name(struct volume_key *vk, const char *key_name);
+key_type_t crypt_volume_key_kernel_key_type(const struct volume_key *vk);
+const char *crypt_volume_key_description(const struct volume_key *vk);
 void crypt_volume_key_set_id(struct volume_key *vk, int id);
 int crypt_volume_key_get_id(const struct volume_key *vk);
 void crypt_volume_key_add_next(struct volume_key **vks, struct volume_key *vk);
 struct volume_key *crypt_volume_key_next(struct volume_key *vk);
 struct volume_key *crypt_volume_key_by_id(struct volume_key *vk, int id);
+void crypt_volume_key_pass_safe_alloc(struct volume_key *vk, void **safe_alloc);
+bool crypt_volume_key_is_set(const struct volume_key *vk);
+bool crypt_volume_key_upload_kernel_key(struct volume_key *vk);
+void crypt_volume_key_drop_uploaded_kernel_key(struct crypt_device *cd, struct volume_key *vk);
+void crypt_volume_key_drop_kernel_key(struct crypt_device *cd, struct volume_key *vk);
 
 struct crypt_pbkdf_type *crypt_get_pbkdf(struct crypt_device *cd);
 int init_pbkdf_type(struct crypt_device *cd,
@@ -86,7 +98,6 @@ int device_alloc_no_check(struct device **device, const char *path);
 void device_close(struct crypt_device *cd, struct device *device);
 void device_free(struct crypt_device *cd, struct device *device);
 const char *device_path(const struct device *device);
-const char *device_dm_name(const struct device *device);
 const char *device_block_path(const struct device *device);
 void device_topology_alignment(struct crypt_device *cd,
 			       struct device *device,
@@ -104,6 +115,7 @@ int device_is_identical(struct device *device1, struct device *device2);
 int device_is_rotational(struct device *device);
 int device_is_dax(struct device *device);
 int device_is_zoned(struct device *device);
+int device_is_nop_dif(struct device *device, uint32_t *tag_size);
 size_t device_alignment(struct device *device);
 int device_direct_io(const struct device *device);
 int device_fallocate(struct device *device, uint64_t size);
@@ -155,6 +167,7 @@ char *crypt_lookup_dev(const char *dev_id);
 int crypt_dev_is_rotational(int major, int minor);
 int crypt_dev_is_dax(int major, int minor);
 int crypt_dev_is_zoned(int major, int minor);
+int crypt_dev_is_nop_dif(int major, int minor, uint32_t *tag_size);
 int crypt_dev_is_partition(const char *dev_path);
 char *crypt_get_partition_device(const char *dev_path, uint64_t offset, uint64_t size);
 int crypt_dev_get_partition_number(const char *dev_path);
@@ -162,8 +175,6 @@ char *crypt_get_base_device(const char *dev_path);
 uint64_t crypt_dev_partition_offset(const char *dev_path);
 int lookup_by_disk_id(const char *dm_uuid);
 int lookup_by_sysfs_uuid_field(const char *dm_uuid);
-int crypt_uuid_cmp(const char *dm_uuid, const char *hdr_uuid);
-int crypt_uuid_type_cmp(const char *dm_uuid, const char *type);
 
 size_t crypt_getpagesize(void);
 unsigned crypt_cpusonline(void);
@@ -217,7 +228,7 @@ int crypt_wipe_device(struct crypt_device *cd,
 
 /* Internal integrity helpers */
 const char *crypt_get_integrity(struct crypt_device *cd);
-int crypt_get_integrity_key_size(struct crypt_device *cd);
+int crypt_get_integrity_key_size(struct crypt_device *cd, bool dm_compat);
 int crypt_get_integrity_tag_size(struct crypt_device *cd);
 
 int crypt_key_in_keyring(struct crypt_device *cd);
@@ -231,9 +242,18 @@ int crypt_keyring_get_key_by_name(struct crypt_device *cd,
 		const char *key_description,
 		char **key,
 		size_t *key_size);
+
+int crypt_keyring_get_keysize_by_name(struct crypt_device *cd,
+		const char *key_description,
+		size_t *r_key_size);
+
 int crypt_use_keyring_for_vk(struct crypt_device *cd);
-void crypt_drop_keyring_key_by_description(struct crypt_device *cd, const char *key_description, key_type_t ktype);
-void crypt_drop_keyring_key(struct crypt_device *cd, struct volume_key *vks);
+void crypt_unlink_key_from_thread_keyring(struct crypt_device *cd,
+		key_serial_t key_id);
+void crypt_unlink_key_by_description_from_thread_keyring(struct crypt_device *cd,
+		const char *key_description,
+		key_type_t ktype);
+void crypt_drop_uploaded_keyring_key(struct crypt_device *cd, struct volume_key *vks);
 
 static inline uint64_t compact_version(uint16_t major, uint16_t minor, uint16_t patch, uint16_t release)
 {
@@ -266,4 +286,6 @@ static inline bool uint64_mult_overflow(uint64_t *u, uint64_t b, size_t size)
 #define KEY_EXTERNAL_VERIFICATION -1
 #define KEY_VERIFIED 0
 
+size_t crypt_safe_alloc_size(const void *data);
+
 #endif /* INTERNAL_H */
