@@ -2,8 +2,8 @@
 /*
  * LUKS - Linux Unified Key Setup v2, internal segment handling
  *
- * Copyright (C) 2018-2024 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2018-2024 Ondrej Kozina
+ * Copyright (C) 2018-2025 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2018-2025 Ondrej Kozina
  */
 
 #include "luks2_internal.h"
@@ -270,7 +270,7 @@ json_object *json_segment_create_linear(uint64_t offset, const uint64_t *length,
 }
 
 static bool json_add_crypt_fields(json_object *jobj_segment, uint64_t iv_offset,
-				  const char *cipher, const char *integrity,
+				  const char *cipher, const char *integrity, uint32_t integrity_key_size,
 				  uint32_t sector_size, unsigned reencryption)
 {
 	json_object *jobj_integrity;
@@ -289,6 +289,8 @@ static bool json_add_crypt_fields(json_object *jobj_segment, uint64_t iv_offset,
 		json_object_object_add(jobj_integrity, "type", json_object_new_string(integrity));
 		json_object_object_add(jobj_integrity, "journal_encryption", json_object_new_string("none"));
 		json_object_object_add(jobj_integrity, "journal_integrity", json_object_new_string("none"));
+		if (integrity_key_size)
+			json_object_object_add(jobj_integrity, "key_size", json_object_new_int(integrity_key_size));
 		json_object_object_add(jobj_segment,   "integrity", jobj_integrity);
 	}
 
@@ -300,7 +302,7 @@ static bool json_add_crypt_fields(json_object *jobj_segment, uint64_t iv_offset,
 
 json_object *json_segment_create_crypt(uint64_t offset,
 				  uint64_t iv_offset, const uint64_t *length,
-				  const char *cipher, const char *integrity,
+				  const char *cipher, const char *integrity, uint32_t integrity_key_size,
 				  uint32_t sector_size, unsigned reencryption)
 {
 	json_object *jobj = _segment_create_generic("crypt", offset, length);
@@ -308,7 +310,7 @@ json_object *json_segment_create_crypt(uint64_t offset,
 	if (!jobj)
 		return NULL;
 
-	if (json_add_crypt_fields(jobj, iv_offset, cipher, integrity, sector_size, reencryption))
+	if (json_add_crypt_fields(jobj, iv_offset, cipher, integrity, integrity_key_size, sector_size, reencryption))
 		return jobj;
 
 	json_object_put(jobj);
@@ -350,7 +352,7 @@ json_object *json_segment_create_opal_crypt(uint64_t offset, const uint64_t *len
 
 	json_add_opal_fields(jobj, length, segment_number, key_size);
 
-	if (json_add_crypt_fields(jobj, iv_offset, cipher, integrity, sector_size, reencryption))
+	if (json_add_crypt_fields(jobj, iv_offset, cipher, integrity, 0, sector_size, reencryption))
 		return jobj;
 
 	json_object_put(jobj);
