@@ -11,14 +11,14 @@
 # $1 full target dir
 # $2 full source luks2 image
 
-function generate()
+generate()
 {
 	CHKS1=$(echo "Arbitrary chosen string: D'oh!" | calc_sha256_checksum_stdin)
 	write_checksum $CHKS1 $TMPDIR/hdr1
 	write_luks2_bin_hdr1 $TMPDIR/hdr1 $TGT_IMG
 }
 
-function check()
+check()
 {
 	lib_hdr1_checksum || exit 2
 }
